
#include "SDUGeneral.h"
#include <Wingdi.h>
#include <commctrl.h>  // Only used for LVS_TYPEMASK
#include <stdio.h>
#include <wchar.h>

// =========================================================================
void SDUParsePath(
    const WCHAR* fullPath, 
    WCHAR** uncMachine,
    WCHAR** drive,
    WCHAR** path,
    WCHAR** filename,
    WCHAR** fileExtension
)
{
    WCHAR* pos;
    int i;
    int tmpLen;

    *uncMachine    = NULL;
    *drive         = NULL;
    *path          = NULL;
    *filename      = NULL;
    *fileExtension = NULL;


    // Identify UNC path/drive letter and path
    if (wcslen(fullPath) >= 3)
        {
        // Check if starts with "\\"; UNC path
        // e.g. \\<machine>\path\...
        if (wcsncmp(fullPath, TEXT("\\\\"), 2) == 0)
            {
            *uncMachine = (WCHAR*)&(fullPath[2]);

            // Seek following path...
            pos = &((*uncMachine)[1]); // Skip "\\"
            while (wcsncmp(pos, TEXT("\0"), 1) != 0)
                {
                if (
                    (wcsncmp(pos, TEXT("\\"), 1) == 0) ||
                    (wcsncmp(pos, TEXT("/"), 1) == 0)
                   )
                    {
                    *path = pos;
                    break;
                    }

                pos++;
                }
            }
        // Check if second char is ":"; DOS drive letter
        // e.g. C:\path\...
        else if (wcsncmp(&(fullPath[1]), TEXT(":"), 1) == 0)
            {
            *drive = (WCHAR*)fullPath;
            *path = (WCHAR*)&(fullPath[2]); // Move past drive letter and ":"
            }
        else 
        // Assume that the input is *just* a path
            {
            *path = (WCHAR*)fullPath;
            }
        }
    else if (wcslen(fullPath) >= 1)
    // Assume that the input is *just* a path
        {
        *path = (WCHAR*)fullPath;
        }


    // Process filename...
    if (*path != NULL)
        {
        // Seek backwards to pick up the filename extension and filename
        tmpLen = wcslen(*path);
        for (i = (tmpLen - 1); i >= 0; i--)
            {
            if (
                (wcsncmp(&((*path)[i]), TEXT("\\"), 1) == 0) ||
                (wcsncmp(&((*path)[i]), TEXT("/"), 1) == 0)
                )
                {
                // If the character we found wasn't the last character, we
                // do have a filename we can use
                if (i != (tmpLen - 1))
                    {
                    *filename = &((*path)[i+1]);
                    }

                break;
                }
            }
        }


    // Process filename extension...
    if (*filename != NULL)
        {
        // Seek backwards to pick up the filename extension and filename
        tmpLen = wcslen(*filename);
        for (i = (tmpLen - 1); i >= 0; i--)
            {
            if (wcsncmp(&((*filename)[i]), TEXT("."), 1) == 0)                 
                {
                // If the character we found wasn't the last character, we
                // do have a filename we can use
                if (i != (tmpLen - 1))
                    {
                    *fileExtension = &((*filename)[i+1]);
                    }

                break;
                }
            }
        }

}


// =========================================================================
void* SDUUnpack_LARGE_INTEGER(const void* ptr, LARGE_INTEGER* val)
{
    void *tmpPtr;
    int i;
    unsigned char x;

    tmpPtr = (void*)ptr;
    val->QuadPart = 0;
    for (i = 0; i < 8; i++)
        {
        tmpPtr = SDUUnpack_char(tmpPtr, (char*)&x);
        val->QuadPart = (val->QuadPart << 8) + x;
        }

    return tmpPtr;
}


// =========================================================================
void* SDUUnpack_DWORD(const void* ptr, DWORD* val)
{
    void *tmpPtr;
    int i;
    unsigned char x;

    tmpPtr = (void*)ptr;
    *val = 0;
    for (i = 0; i < 4; i++)
        {
        tmpPtr = SDUUnpack_char(tmpPtr, (char*)&x);
        *val = (*val << 8) + x;
        }

    return tmpPtr;
}


// =========================================================================
void* SDUUnpack_char(const void* ptr, char* val)
{
    *val = (*((char*)ptr));
    return ((char*)ptr + 1);
}


// =========================================================================
void* SDUPack_LARGE_INTEGER(const void* ptr, LARGE_INTEGER val)
{
    void *tmpPtr;

    tmpPtr = (void *)ptr;

    tmpPtr = SDUPack_DWORD(tmpPtr, val.HighPart);
    tmpPtr = SDUPack_DWORD(tmpPtr, val.LowPart);

    return tmpPtr;
}


// =========================================================================
void* SDUPack_DWORD(const void* ptr, DWORD val)
{
    void *tmpPtr;
    unsigned char x;

    tmpPtr = (void *)ptr;

    x = (unsigned char)((val >> 24) & 0xFF);
    tmpPtr = SDUPack_char(tmpPtr, (char)x);
    x = (unsigned char)((val >> 16) & 0xFF);
    tmpPtr = SDUPack_char(tmpPtr, (char)x);
    x = (unsigned char)((val >> 8) & 0xFF);
    tmpPtr = SDUPack_char(tmpPtr, (char)x);
    x = (unsigned char)(val & 0xFF);
    tmpPtr = SDUPack_char(tmpPtr, (char)x);

    return tmpPtr;
}


// =========================================================================
void* SDUPack_char(const void* ptr, char val)
{
    *((char*)ptr) = val;
    return ((char*)ptr + 1);
}


// =========================================================================
// Set filename to NULL to get version info on the current process
// Note: Caller must free(lpBuffer) after use
BOOL SDUGetVersionInfoShortFmtAlloc(
					   LPWSTR filename,
					   LPWSTR* lpBuffer
					  )
{
    BOOL retval = FALSE;
    const int VERSION_STR_SIZE = 256;

    *lpBuffer = (LPWSTR)malloc(VERSION_STR_SIZE * sizeof(*lpBuffer));
    if (SDUGetVersionInfoShortFmt(
                             filename,
                             *lpBuffer,
                             VERSION_STR_SIZE
                             ))
        {
        retval = TRUE;
        }
    else
        {
        free(*lpBuffer);
        *lpBuffer = NULL;
        }

    return retval;
}


// =========================================================================
// Set filename to NULL to get version info on the current process
// Note: nSize must be the max number of *characters*
BOOL SDUGetVersionInfoShortFmt(
					   LPWSTR filename,
					   LPWSTR lpBuffer,
					   DWORD nSize
					  )
{
    BOOL retval = FALSE;
    DWORD Version;

    if (SDUGetVersionInfoShort(filename, &Version))
        {
        if (_snwprintf(
                       lpBuffer, 
                       nSize,
                       TEXT("v%d.%0.2d.%0.4d"), 
                       (Version & 0xFF000000) / 0x00FF0000,
                       (Version & 0x00FF0000) / 0x0000FF00,
                       (Version & 0x0000FFFF)
                      ) >= 0)
/*
        if (_snwprintf_(
                       lpBuffer, 
                       nSize,
                       nSize,
                       TEXT("v%d.%0.2d.%0.4d"), 
                       (Version & 0xFF000000) / 0x00FF0000,
                       (Version & 0x00FF0000) / 0x0000FF00,
                       (Version & 0x0000FFFF)
                      ) >= 0)
*/
            {
            retval = TRUE;
            }
        }

    return retval;   
}


// =========================================================================
// Set filename to NULL to get version info on the current process
// Note: Caller must free(lpBuffer)  after use
BOOL SDUGetVersionInfoFmtAlloc(
					   LPWSTR filename,
					   LPWSTR* lpBuffer
					  )
{
    BOOL retval = FALSE;
    const int VERSION_STR_SIZE = 256;

    *lpBuffer = (LPWSTR)malloc(VERSION_STR_SIZE);
    if (SDUGetVersionInfoFmt(
                             filename,
                             *lpBuffer,
                             VERSION_STR_SIZE
                             ))
        {
        retval = TRUE;
        }
    else
        {
        free(*lpBuffer);
        *lpBuffer = NULL;
        }

    return retval;
}


// =========================================================================
// Set filename to NULL to get version info on the current process
BOOL SDUGetVersionInfoFmt(
					   LPWSTR filename,
					   LPWSTR lpBuffer,
					   DWORD nSize
					  )
{
    BOOL retval = FALSE;
    int majorVersion;
    int minorVersion;
    int revisionVersion;
    int buildVersion;

    if (SDUGetVersionInfo(
                          filename,
                          &majorVersion,
                          &minorVersion, 
                          &revisionVersion, 
                          &buildVersion
                         ))
        {
        if (_snwprintf(
                       lpBuffer, 
                       nSize,
                       TEXT("v%d.%0.2d.%0.2d.%0.4d"), 
                       majorVersion, 
                       minorVersion, 
                       revisionVersion, 
                       buildVersion
                      ) >= 0)
/*
        if (_snwprintf_s(
                       lpBuffer, 
                       nSize,
                       nSize,
                       TEXT("v%d.%0.2d.%0.2d.%0.4d"), 
                       majorVersion, 
                       minorVersion, 
                       revisionVersion, 
                       buildVersion
                      ) >= 0)
*/
            {
            retval = TRUE;
            }
        }

    return retval;   
}


// =========================================================================
// Set filename to NULL to get version info on the current process
// This sets Version to the version ID in the form: 0xAABBCCCC
// Where: 
//   AA - Major version
//   BB - Minor version
//   CC - Revision version
BOOL SDUGetVersionInfoShort(
					   LPWSTR Filename,
					   DWORD* Version
					  )
{
    BOOL retval;
    int majorVersion;
    int minorVersion;
    int revisionVersion;
    int buildVersion;

    retval = SDUGetVersionInfo(
                               Filename,
                               &majorVersion,
                               &minorVersion,
                               &revisionVersion,
                               &buildVersion
                              );
    if (retval)
        {
        *Version = (
                    (majorVersion << 24) +
                    (minorVersion << 16) +
                    revisionVersion
                   );
        }

    return retval;
}


// =========================================================================
// Set filename to NULL to get version info on the current process
BOOL SDUGetVersionInfo(
					   LPWSTR filename,
					   int* majorVersion,
					   int* minorVersion, 
					   int* revisionVersion, 
					   int* buildVersion
					  )
{
    BOOL retval = FALSE;
    int vsize;
    UINT puLen;
    DWORD dwHandle;
    void* pBlock;
    void* pVPointer = NULL;
    VS_FIXEDFILEINFO* tvs;
    // Twice MAX_PATH should be sufficient for most purposes...
    WCHAR bufFilename[sizeof(WCHAR) * (2 * MAX_PATH)];

    // Autodetermine filename if not passed in
    if (filename == NULL)
        {
        if (GetModuleFileName( 
                              NULL,
                              bufFilename, 
                              sizeof(bufFilename)
                             ) != 0)
            {
            filename = bufFilename;
            }
        }

    if (filename != NULL)
        {
        vsize = GetFileVersionInfoSize(filename, &dwHandle);
        if (vsize > 0) 
            {
            pBlock = malloc(vsize);

            if (GetFileVersionInfo(filename, dwHandle, vsize, pBlock))
                {
                if (VerQueryValue(pBlock, _T("\\"), &pVPointer, &puLen))
                    {
                    if (puLen > 0)
                        {
                        tvs = (VS_FIXEDFILEINFO*)pVPointer;
                        *majorVersion    = tvs->dwFileVersionMS >> 16;
                        *minorVersion    = tvs->dwFileVersionMS & 0xFFFF;
                        *revisionVersion = tvs->dwFileVersionLS >> 16;
                        *buildVersion    = tvs->dwFileVersionLS & 0xFFFF;
                        retval = TRUE;
                        }
                    }
                }

            free(pBlock);
            }
        }

    return retval;
}


// =========================================================================
void SDUTextSetBold(
    HWND hWnd,
    int ctrlID,
    BOOL UseBold
)
{
    HFONT hFont;
    LOGFONT lFont;

    // Set welcome title to bold
    hFont = (HFONT)SendMessage(
                GetDlgItem(hWnd, ctrlID),
                WM_GETFONT,
                0,
                0
               );
    if (hFont != NULL)
        {
        if (GetObject(hFont, sizeof(lFont), &lFont) != 0)
            {
            lFont.lfWeight = FW_BOLD;
            hFont = CreateFontIndirect(&lFont);
            SendMessage(
                        GetDlgItem(hWnd, ctrlID),
                        WM_SETFONT,
                        (WPARAM)hFont,
                        TRUE
                       );
            }
        }
}


// =========================================================================
void SDUTextSetItalic(
    HWND hWnd,
    int ctrlID,
    BOOL UseItalics
)
{
    HFONT hFont;
    LOGFONT lFont;

    // Set welcome title to bold
    hFont = (HFONT)SendMessage(
                GetDlgItem(hWnd, ctrlID),
                WM_GETFONT,
                0,
                0
               );
    if (hFont != NULL)
        {
        if (GetObject(hFont, sizeof(lFont), &lFont) != 0)
            {
            lFont.lfItalic = UseItalics;
            hFont = CreateFontIndirect(&lFont);
            SendMessage(
                        GetDlgItem(hWnd, ctrlID),
                        WM_SETFONT,
                        (WPARAM)hFont,
                        TRUE
                       );
            }
        }
}


// =========================================================================
void SDUTextSetUnderline(
    HWND hWnd,
    int ctrlID,
    BOOL UseUnderline
)
{
    HFONT hFont;
    LOGFONT lFont;

    // Set welcome title to bold
    hFont = (HFONT)SendMessage(
                GetDlgItem(hWnd, ctrlID),
                WM_GETFONT,
                0,
                0
               );
    if (hFont != NULL)
        {
        if (GetObject(hFont, sizeof(lFont), &lFont) != 0)
            {
            lFont.lfUnderline = UseUnderline;
            hFont = CreateFontIndirect(&lFont);
            SendMessage(
                        GetDlgItem(hWnd, ctrlID),
                        WM_SETFONT,
                        (WPARAM)hFont,
                        TRUE
                       );
            }
        }
}


// =========================================================================
// Draw text as per DrawText(...), but truncate the string displayed and
// append "..." onto it if it will not fit into the boundrys supplied
// This is implemented for systems where DT_WORD_ELLIPSIS and DT_PATH_ELLIPSIS
// are not supported (e.g. WinCE)
// Returns: The height of the text displayed
#define SDU_ELLIPSES  TEXT("...")
int SDUDrawTextEllipses(
    HDC hdc,
    WCHAR* Text,
    int TextLength,
    LPRECT TextRect,
    UINT uFormat
)
{
    int retval;
    SIZE textSize;
    int bufferSize;
    WCHAR* buffer;
    int useTextLength;

    retval = 0;

    // +1 for NULL terminator
    bufferSize = (wcslen(Text) + wcslen(SDU_ELLIPSES) + 1) * sizeof(*buffer);
    buffer = (WCHAR*)malloc(bufferSize);
    if (buffer != NULL)
        {
        useTextLength = TextLength;
        memset(buffer, 0, bufferSize);
        wcsncpy(
                  buffer, 
                  Text,
                  useTextLength
                 );
/*
        wcsncpy_s(
                  buffer, 
                  (bufferSize / sizeof(*buffer)),
                  Text,
                  useTextLength
                 );
*/

        GetTextExtentPoint32(
                             hdc, 
                             buffer,
                             wcslen(buffer),
                             &textSize
                            );
        if (textSize.cx > (TextRect->right - TextRect->left))
            {
            wcscat(buffer, SDU_ELLIPSES);
//            wcscat_s(buffer, (bufferSize / sizeof(WORD)), SDU_ELLIPSES);
            GetTextExtentPoint32(
                                 hdc, 
                                 buffer,
                                 wcslen(buffer),
                                 &textSize
                                );
            }

        while (
               (textSize.cx > (TextRect->right - TextRect->left)) &&
               (useTextLength > 0)
              )
            {
            useTextLength--;
            memset(buffer, 0, bufferSize);
            wcsncpy(
                      buffer, 
                      Text, 
                      useTextLength
                     );
            wcscat(buffer, SDU_ELLIPSES);
/*
            wcsncpy_s(
                      buffer, 
                      (bufferSize / sizeof(WORD)), 
                      Text, 
                      useTextLength
                     );
            wcscat_s(buffer, (bufferSize / sizeof(WORD)), SDU_ELLIPSES);
*/
            GetTextExtentPoint32(
                                 hdc, 
                                 buffer,
                                 wcslen(buffer),
                                 &textSize
                                );
            }

        retval = DrawText(
                          hdc,
                          buffer,
                          wcslen(buffer),
                          TextRect,
                          uFormat
                         );

        free(buffer);
        }

    return retval;
}


// =========================================================================
// Set listview style; one of:
//   LVS_ICON      - Large icons with text underneath them. Bit like Windows
//                   desktop icons in layout
//   LVS_REPORT    - Grid with header. Subitems displayed as separate columns.
//                   This is the only one which can do full row select
//   LVS_SMALLICON - Just a list, no header. Only first item (no subitems)
//                   displayed
//   LVS_LIST      - Just a list, no header. Only first item (no subitems)
//                   displayed
// To get the listview to display a report view with large icons, supply
// normal sized (i.e. 32x32) icons as the listview's "small icons" imagelist
void SDUSetListViewType(HWND hWnd, DWORD newStyle)
{
    DWORD style;

    style = GetWindowLong(hWnd, GWL_STYLE);

    if((style & LVS_TYPEMASK) != newStyle)
        {
        SetWindowLong(
                      hWnd, 
                      GWL_STYLE, 
                      ((style & ~LVS_TYPEMASK) | newStyle)
                     );
        } 

}


// =========================================================================
// SetNotUnset - Set to TRUE to set the style specified, FALSE to remove it
void SDUSetWndStyle(HWND hWnd, BOOL SetNotUnset, DWORD Style)
{
    DWORD wndStyle;

    wndStyle = GetWindowLong(hWnd, GWL_STYLE);

    if (SetNotUnset)
        {
        wndStyle |= Style;
        }
    else
        {
        wndStyle &= ~Style;
        }

    SetWindowLong(
                  hWnd, 
                  GWL_STYLE, 
                  wndStyle
                 );
}


// =========================================================================
// The one MS missed out... Get the length of a listview items's text in
// characters
int SDUListView_GetItemTextLength(HWND hWnd, int item, int subItem)
{
    LVITEM listItem;
    WCHAR* buffer;
    int testCharCount;
    int v;

    memset(&listItem, 0, sizeof(listItem));
    listItem.iItem = item;
    listItem.iSubItem = subItem;
    listItem.mask = LVIF_TEXT;

    testCharCount = 5;
    buffer = (WCHAR*)malloc((testCharCount + 1) * sizeof(*buffer));
    listItem.pszText = buffer;
    listItem.cchTextMax = testCharCount;
    v = SendMessage(hWnd, LVM_GETITEMTEXT, item, (LPARAM)&listItem);

    testCharCount = 1;
    while (v >= (testCharCount - 1))
        {
        testCharCount += 5;
        buffer = (WCHAR*)realloc(buffer, (testCharCount + 1) * sizeof(*buffer));
        listItem.pszText = buffer;
        listItem.cchTextMax = testCharCount;
        v = SendMessage(hWnd, LVM_GETITEMTEXT, item, (LPARAM)&listItem);
        }

    if (buffer != NULL)
        {
        free(buffer);
        }

    return v; // listItem.cchTextMax;
}


// =========================================================================
// Tokenize a command line and it's parameters
// Analagous to strtok(...) in it's operation
WCHAR* _SDUParamTok(WCHAR* cmdLine)
{
    static WCHAR* ptr;
    WCHAR* start;
    WCHAR* end;
    WCHAR* tmp;
    WCHAR endOfParamChar;

    if (cmdLine != NULL)
        {
        ptr = cmdLine;
        }

    if (ptr == NULL)
        {
        return NULL;
        }

    start = NULL;
    tmp = ptr;
    while (*tmp != 0)
        {
        if (*tmp != ' ')
            {
            start = tmp;
            break;
            }

        tmp++;
        }

    if (start == NULL)
        {
        ptr = NULL;
        return NULL;
        }

    endOfParamChar = ' ';
    if (*start == '"')
        {
        start = &(start[1]);
        endOfParamChar = '"';    
        }

    end = start;
    tmp = start;
    while (
           (*end != 0) &&
           (*end != endOfParamChar)
           )
        {
        end++;
        }

    if (*end == 0)
        {
        ptr = NULL;
        }
    else
        {
        ptr = &(end[1]);
        }

    if (
        (start == end) &&
        (endOfParamChar != '"')
       )
        {
        ptr = NULL;
        start = NULL;
        }

    *end = 0;

    return start;
}


// =========================================================================
// Return the value of the command line parameter indicated by the specified
// index (zero based index)
// IMPORTANT: It is the *callers* responsibility to free off the value returned
// Returns NULL on failure
WCHAR* SDUCommandLineParameter_Idx(const int parameterIdx)
{
    WCHAR* origCmdLine;
    WCHAR* useCmdLine;
    WCHAR* currParam;
    int currParamIdx;
    WCHAR* retval;

    retval = NULL;

    origCmdLine = GetCommandLine();
    // +1 to include NULL terminator
    useCmdLine = calloc((wcslen(origCmdLine) + 1), sizeof(*useCmdLine));
    if (useCmdLine != NULL)
        {
        wcscpy(useCmdLine, origCmdLine);

        currParamIdx = 0;
        currParam = _SDUParamTok(useCmdLine);
        while (
               (currParamIdx < parameterIdx) &&
               (currParam != NULL)
              )
            {
            currParamIdx++;
            currParam = _SDUParamTok(NULL);
            }

        if (currParam != NULL)
            {
            retval = calloc((wcslen(currParam) + 1), sizeof(*retval));
            if (retval != NULL)
                {
                wcscpy(retval, currParam);
                }
            }

        free(useCmdLine);
        }

    return retval;
}

// =========================================================================
// Returns TRUE if the specified command line switch could be found, otherwise
// FALSE
BOOL SDUCommandLineSwitch(const WCHAR* parameter)
{
    BOOL retval;

    retval = FALSE;

    if (SDUCommandLineSwitchNumber(parameter) >= 0)
        {
        retval = TRUE;
        }

    return retval;
}



// =========================================================================
// Return the value of the specified command line parameter "-<parameter>
// value"
// IMPORTANT: It is the *callers* responsibility to free off the value returned
// Returns NULL on failure
WCHAR* SDUCommandLineParameter_Name(const WCHAR* parameter)
{
    WCHAR* retval;
    int switchParamIdx;

    retval = NULL;

    switchParamIdx = SDUCommandLineSwitchNumber(parameter);
    if (switchParamIdx >= 0)
        {
        retval = SDUCommandLineParameter_Idx(switchParamIdx+1);
        }

    return retval;
}


// =========================================================================
// Returns the parameter number in the command line of the specified parameter.
// Returns -1 on failure
int SDUCommandLineSwitchNumber(const WCHAR* parameter)
{
    int retval;
    WCHAR* currParam;
    int currParamIdx;

    retval = -1;

    currParamIdx = 0;
    currParam = SDUCommandLineParameter_Idx(currParamIdx);
    while (currParam != NULL)
        {
        // >2 to allow 
        if (wcslen(currParam) > 1)
            {
            if (
                (currParam[0] == '/') ||
                (currParam[0] == '-')
               )
                {
                if (wcscmp(parameter, &(currParam[1])) == 0)
                    {
                    retval = currParamIdx;
                    break;
                    }
                }
            }

        free(currParam);                    
        currParamIdx++;
        currParam = SDUCommandLineParameter_Idx(currParamIdx);
        }

    if (currParam != NULL)
        {
        free(currParam);
        }

    return retval;
}


// =========================================================================
// Identify which version of Windows Mobile is running
SDU_OS_VER_PDA SDUGetOSVersion_PDA()
{
    SDU_OS_VER_PDA retval;
    OSVERSIONINFO osVer;

    retval = SDU_OS_VER_PDA_UNKNOWN;

    //  VER_PLATFORM_WIN32_CE
    if (GetVersionEx(&osVer))
        {
        if (
            (osVer.dwMajorVersion >= 5) &&
            (osVer.dwMinorVersion >= 2)
           )
            {
            // aka WM6
            retval = SDU_OS_VER_PDA_CROSSBOW;
            }
        else if (osVer.dwMajorVersion >= 5)
            {
            // aka WM5
            retval = SDU_OS_VER_PDA_WM2005;
            }
        else if (
            (osVer.dwMajorVersion >= 4) &&
            (osVer.dwMinorVersion >= 2)
           )
            {
            retval = SDU_OS_VER_PDA_WM2003SE;
            }
        else if (osVer.dwMajorVersion >= 4)
            {
            retval = SDU_OS_VER_PDA_WM2003;
            }
        else if (osVer.dwMajorVersion >= 3)
            {
            retval = SDU_OS_VER_PDA_WM2002;  // SDU_OS_VER_PDA_CE30;
            }
        else if (
                 (osVer.dwMajorVersion >= 2) &&
                 (osVer.dwMinorVersion >= 1)
                )
            {
            retval = SDU_OS_VER_PDA_CE21;
            }
        else if (osVer.dwMajorVersion >= 2)
            {
            retval = SDU_OS_VER_PDA_CE20;
            }
        else 
            {
            retval = SDU_OS_VER_PDA_CE10;
            }
        }

    return retval;
}

// =========================================================================
// =========================================================================

