unit FreeOTFEfrmOptions;
// Description:
// By Sarah Dean
// Email: sdean12@sdean12.org
// WWW:   http://www.FreeOTFE.org/
//
// -----------------------------------------------------------------------------
//


interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ComCtrls,
  FreeOTFESettings, FreeOTFEfmeOptions_SystemTray, FreeOTFEfmeOptions_Hotkeys,
  FreeOTFEfmeOptions_Base, FreeOTFEfmeOptions_General,
  FreeOTFEfmeOptions_AutoRun;

type
  TfrmOptions = class(TForm)
    pbOK: TButton;
    pbCancel: TButton;
    cbSettingsLocation: TComboBox;
    lblSettingsLocation: TLabel;
    ckAssociateFiles: TCheckBox;
    pcOptions: TPageControl;
    tsGeneral: TTabSheet;
    tsHotkeys: TTabSheet;
    fmeOptions_General1: TfmeOptions_General;
    fmeOptions_Hotkeys1: TfmeOptions_Hotkeys;
    tcSystemTray: TTabSheet;
    fmeOptions_SystemTray1: TfmeOptions_SystemTray;
    tsAutorun: TTabSheet;
    fmeOptions_Autorun1: TfmeOptions_Autorun;
    procedure pbOKClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure pbCancelClick(Sender: TObject);
    procedure ControlChanged(Sender: TObject);
  private
    FOrigAssociateFiles: boolean;
    
    procedure EnableDisableControls();
    function  GetSettingsLocation(): TSettingsSaveLocation;
  public
    { Public declarations }
  end;

implementation

{$R *.DFM}

uses
  ShlObj,  // Required for CSIDL_PROGRAMS
  SDUGeneral,
  SDUDialogs,
  OTFEFreeOTFE_U;

const
  SHELL_VERB = 'Open';

function TfrmOptions.GetSettingsLocation(): TSettingsSaveLocation;
var
  retval: TSettingsSaveLocation;
  sl: TSettingsSaveLocation;
begin
  retval := Settings.OptSaveSettings;
  for sl:=low(SettingsSaveLocationTitle) to high(SettingsSaveLocationTitle) do
    begin
    if (SettingsSaveLocationTitle[sl] = cbSettingsLocation.Items[cbSettingsLocation.ItemIndex]) then
      begin
      retval := sl;
      break;
      end;
    end;

  Result := retval;
end;

procedure TfrmOptions.pbOKClick(Sender: TObject);
var
  allOK: boolean;
  oldSettingsLocation: TSettingsSaveLocation;
  newSettingsLocation: TSettingsSaveLocation;
  msgSegment: string;
  vistaProgFilesDirWarn: boolean;
  programFilesDir: string;
  filename: string;
  prevSDUDialogsStripSingleCRLF: boolean;
  i: integer;
  j: integer;
  currTabSheet: TTabSheet;
begin
  allOK := TRUE;

  // Decode settings save location
  oldSettingsLocation := Settings.OptSaveSettings;
  newSettingsLocation := GetSettingsLocation();

  if allOK then
    begin
    // If user tried to save settings under C:\Program Files\... while
    // running under Vista, let the user know that Vista's security
    // system screws this up (it maps the save to somewhere in the user's
    // profile)
    vistaProgFilesDirWarn:= FALSE;
    if (
        SDUOSVistaOrLater() and
        (newSettingsLocation = slExeDir)
       ) then
      begin
      programFilesDir := SDUGetSpecialFolderPath(SDU_CSIDL_PROGRAM_FILES);
      filename := Settings.GetSettingsFilename(newSettingsLocation);
      vistaProgFilesDirWarn := (Pos(
                                    uppercase(programFilesDir),
                                    uppercase(filename)
                                   ) > 0);
      end;

    allOK := not(vistaProgFilesDirWarn);

    if not(allOK) then
      begin
      prevSDUDialogsStripSingleCRLF := SDUDialogsStripSingleCRLF;  // Don't do special processing on this message
      SDUDialogsStripSingleCRLF := FALSE;  // Don't do special processing on this message
      SDUMessageDlg(
                    'Under Windows Vista, you cannot save your settings file anywhere under:'+SDUCRLF+
                    SDUCRLF+
                    programFilesDir+SDUCRLF+
                    SDUCRLF+
                    'due to Vista''s security/mapping system.'+SDUCRLF+
                    SDUCRLF+
                    'Please either select another location for storing your settings (e.g. your user profile), or move the FreeOTFE executable such that it is not stored underneath the directory shown above.',
                    mtWarning,
                    [mbOK],
                    0
                   );
      SDUDialogsStripSingleCRLF := prevSDUDialogsStripSingleCRLF;  // Don't do special processing on this message
      end;

    end;

  if allOK then
    begin
    Settings.OptSaveSettings := newSettingsLocation;

    // For each tab, scan it's controls; if it's one of our frames, get it to
    // process...
    for i:=0 to (pcOptions.PageCount - 1) do
      begin
      currTabSheet := pcOptions.Pages[i];
      for j:=0 to (currTabSheet.Controlcount - 1) do
        begin
        if (currTabSheet.Controls[j] is TfmeOptions_Base) then
          begin
          TfmeOptions_Base(currTabSheet.Controls[j]).WriteSettings();
          end;
        end;
      end;

    if (Settings.OptSaveSettings <> slNone) then
      begin
      allOK := Settings.Save();
      end;

    end;

  if allOK then
    begin
    // If the save settings location has been changed, and we previously
    // saved settings to a file, ask the user if they want to delete their
    // old settings file
    if (
        (oldSettingsLocation <> newSettingsLocation) and
        (oldSettingsLocation <> slNone)
       ) then
      begin
      if (newSettingsLocation = slNone) then
        begin
        msgSegment := 'You have opted not to save your settings';
        end
      else
        begin
        msgSegment := 'You have changed the location where your settings will be saved';
        end;

      if (SDUMessageDlg(
                    msgSegment+SDUCRLF+
                    SDUCRLF+
                    'Would you like FreeOTFE to delete your previous settings file?',
                    mtConfirmation,
                    [mbYes, mbNo],
                    0
                   ) = mrYes) then
        begin
        if not(Settings.DestroySettingsFile(oldSettingsLocation)) then
          begin
          SDUMessageDlg(
                        'Your previous settings file stored at:'+SDUCRLF+
                        SDUCRLF+
                        Settings.GetSettingsFilename(oldSettingsLocation)+SDUCRLF+
                        SDUCRLF+
                        'could not be deleted. Please remove this file manually.',
                        mtInformation,
                        [mbOK],
                        0
                       );
          end;
        end
      else
        begin
        SDUMessageDlg(
                      'Your previous settings will remain stored at:'+SDUCRLF+
                      SDUCRLF+
                      Settings.GetSettingsFilename(oldSettingsLocation),
                      mtInformation,
                      [mbOK],
                      0
                     );
        end;
      end;
    end;

  if allOK then
    begin
    if (ckAssociateFiles.checked <> FOrigAssociateFiles) then
      begin
      if ckAssociateFiles.checked then
        begin
        SDUFileExtnRegCmd(
                          VOL_FILE_EXTN,
                          SHELL_VERB,
                          '"'+ParamStr(0)+'" /mount /volume "%1"'
                         );
        SDUFileExtnRegIcon(
                           VOL_FILE_EXTN,
                           ParamStr(0),
                           0
                          );
        end
      else
        begin
        SDUFileExtnUnregIcon(VOL_FILE_EXTN);
        SDUFileExtnUnregCmd(
                          VOL_FILE_EXTN,
                          SHELL_VERB
                         );
        end;

      FOrigAssociateFiles := ckAssociateFiles.checked;
      end;

    end;

  if allOK then
    begin
    ModalResult := mrOK;
    end;

end;


procedure TfrmOptions.FormCreate(Sender: TObject);
var
  idx: integer;
  useIdx: integer;
  sl: TSettingsSaveLocation;
  i: integer;
  j: integer;
  currTabSheet: TTabSheet;
begin
  // Set active page to the first one
  pcOptions.ActivePage := tsGeneral;

  // Populate and set settings location dropdown
  cbSettingsLocation.Items.Clear();
  idx := -1;
  useIdx := -1;
  for sl := low(SettingsSaveLocationTitle) to high(SettingsSaveLocationTitle) do
    begin
    inc(idx);
    cbSettingsLocation.Items.Add(SettingsSaveLocationTitle[sl]);
    if (Settings.OptSaveSettings = sl) then
      begin
      useIdx := idx;
      end;
    end;
  cbSettingsLocation.ItemIndex := useIdx;

  // For each tab, scan it's controls; if it's one of our frames, get it to
  // process...
  for i:=0 to (pcOptions.PageCount - 1) do
    begin
    currTabSheet := pcOptions.Pages[i];
    for j:=0 to (currTabSheet.Controlcount - 1) do
      begin
      if (currTabSheet.Controls[j] is TfmeOptions_Base) then
        begin
        TfmeOptions_Base(currTabSheet.Controls[j]).ReadSettings();
        end;
      end;
    end;

  FOrigAssociateFiles := SDUFileExtnIsRegCmd(
                                          VOL_FILE_EXTN,
                                          SHELL_VERB,
                                          ExtractFilename(ParamStr(0))
                                         );
  ckAssociateFiles.checked := FOrigAssociateFiles;

  EnableDisableControls();

end;

procedure TfrmOptions.EnableDisableControls();
const
  CONTROL_MARGIN = 10;
var
  i: integer;
  j: integer;
  currTabSheet: TTabSheet;
begin
  // For each tab, scan it's controls; if it's one of our frames, get it to
  // process...
  for i:=0 to (pcOptions.PageCount - 1) do
    begin
    currTabSheet := pcOptions.Pages[i];
    for j:=0 to (currTabSheet.Controlcount - 1) do
      begin
      if (currTabSheet.Controls[j] is TfmeOptions_Base) then
        begin
        TfmeOptions_Base(currTabSheet.Controls[j]).EnableDisableControls();
        end;
      end;
    end;

  if (GetSettingsLocation() = slNone) then
    begin
    lblSettingsLocation.font.style := [fsBold];
    end
  else
    begin
    lblSettingsLocation.font.style := [];
    end;

  // Adjust "Save settings to:" label
  lblSettingsLocation.Left := (
                               cbSettingsLocation.left -
                               lblSettingsLocation.Width -
                               CONTROL_MARGIN
                              );

end;

procedure TfrmOptions.pbCancelClick(Sender: TObject);
begin
  ModalResult := mrCancel;
  
end;

procedure TfrmOptions.ControlChanged(Sender: TObject);
begin
  EnableDisableControls();
  
end;

END.


